@@ -24,6 +24,7 @@ impl Parse for Args {
     }
 }
 
+#[derive(Debug)]
 struct LLVMPath {
     clang: String,
     llvm_objcopy: String,
@@ -55,26 +56,46 @@ fn get_llvm_path() -> LLVMPath {
 
 #[cfg(not(target_os = "macos"))]
 fn get_llvm_path() -> LLVMPath {
-    use std::path::Path;
-    let which = Command::new("which")
-        .args(["clang-8"])
-        .output()
-        .expect("Failed to run which command");
-    assert!(
-        which.status.success(),
-        "Failed to get llvm path from which: {}",
-        String::from_utf8_lossy(&which.stderr)
-    );
-    let clang_path = String::from_utf8_lossy(&which.stdout).to_string();
-    let base_path = Path::new(&clang_path)
-        .parent()
-        .expect("Failed to take a parent path")
-        .to_str()
-        .expect("Failed to convert from Path to &str")
-        .to_string();
+    let supported_versions = (6..=13).rev();
+    let mut clang_versions: Vec<String> = vec!["clang".to_string()];
+    let mut objcopy_versions: Vec<String> = vec!["llvm-objcopy".to_string()];
+    for v in supported_versions {
+        clang_versions.extend_from_slice(&[
+            format!("clang-{}", v),
+            format!("clang-{}.0", v),
+            format!("clang-{}0", v),
+        ]);
+        objcopy_versions.extend_from_slice(&[
+            format!("llvm-objcopy-{}", v),
+            format!("llvm-objcopy-{}.0", v),
+            format!("llvm-objcopy-{}0", v),
+        ]);
+    }
+
+    let clang_results = clang_versions.iter().map(which::which);
+    let objcopy_results = objcopy_versions.iter().map(which::which);
+    let results = clang_results.zip(objcopy_results);
+    let mut results = results.filter(|x| x.0.is_ok() && x.1.is_ok());
+
+    let path = results
+        .next()
+        .unwrap_or_else(|| panic!("clang or llvm-objcopy not found"));
+
     LLVMPath {
-        clang: base_path.clone() + &"/clang-8".to_string(),
-        llvm_objcopy: base_path + &"/llvm-objcopy-8".to_string(),
+        clang: path.0.unwrap().to_string_lossy().to_string(),
+        llvm_objcopy: path.1.unwrap().to_string_lossy().to_string(),
+    }
+}
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    #[ignore]
+    fn print_llvm_path() {
+        let llvm_path = get_llvm_path();
+        println!("{:?}", llvm_path);
     }
 }
 
@@ -107,7 +128,7 @@ pub fn bingen(input: TokenStream) -> TokenStream {
     let mut input = File::create(dir.path().join("bingen.S")).unwrap();
     input.write_all(asm.value().as_bytes()).unwrap();
 
-    Command::new(clang)
+    let result = Command::new(clang.clone())
         .args([
             "-target",
             &arch.value(),
@@ -125,8 +146,15 @@ pub fn bingen(input: TokenStream) -> TokenStream {
         ])
         .output()
         .expect("Failed to run clang");
+    assert!(
+        result.status.success(),
+        "{} returned {:?}. stderr:\n{}",
+        clang,
+        result.status.code().expect("exit code not set"),
+        String::from_utf8_lossy(&result.stderr)
+    );
 
-    Command::new(llvm_objcopy)
+    let result = Command::new(llvm_objcopy.clone())
         .args([
             "-O",
             "binary",
@@ -143,6 +171,14 @@ pub fn bingen(input: TokenStream) -> TokenStream {
         .output()
         .expect("Failed to run objcopy");
 
+    assert!(
+        result.status.success(),
+        "{} returned {:?}. stderr:\n{}",
+        llvm_objcopy,
+        result.status.code().expect("exit code not set"),
+        String::from_utf8_lossy(&result.stderr)
+    );
+
     format!(
         "{:?}",
         fs::read(dir.path().join("bingen.bin")).expect("Failed to open /tmp/bingen.bin")
